"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StorageService = void 0;
const common_1 = require("@nestjs/common");
const client_s3_1 = require("@aws-sdk/client-s3");
const storage_module_definition_1 = require("./storage.module-definition");
const fs = require("fs");
const lib_storage_1 = require("@aws-sdk/lib-storage");
const stream = require("stream");
let StorageService = class StorageService {
    constructor(options) {
        this.options = options;
        this.useFileSystem = false;
        switch (options.type) {
            case 's3':
                this.endpoint = options.endpoint;
                this.region = options.region;
                this.bucket = options.bucket;
                this.accessKeyId = options.accessKeyId;
                this.secretAccessKey = options.secretAccessKey;
                break;
            case 'fileSystem':
            default:
                this.useFileSystem = true;
                break;
        }
    }
    get client() {
        return this.s3Client;
    }
    get useCloud() {
        return !this.useFileSystem;
    }
    createTest(test, location) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const [key, data] of Object.entries(test)) {
                if (typeof data === 'string') {
                    yield this.writeFile(this.normalizeKey(location + key), data);
                }
                else {
                    yield this.mkdir(this.normalizeDirKey(location + key));
                    yield this.createTest(data, this.normalizeDirKey(location + key));
                }
            }
        });
    }
    readTest(test, location) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const [key, data] of Object.entries(test)) {
                if (typeof data === 'string') {
                }
                else {
                    yield this.readTest(data, this.normalizeDirKey(location + key));
                }
            }
        });
    }
    deleteTest(test, location) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const [key, data] of Object.entries(test)) {
                if (typeof data === 'string') {
                    yield this.rm(this.normalizeKey(location + key));
                }
                else {
                    yield this.deleteTest(data, this.normalizeDirKey(location + key));
                    yield this.rmdir(this.normalizeDirKey(location + key));
                }
            }
        });
    }
    onModuleInit() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.useFileSystem && this.endpoint && this.region && this.accessKeyId && this.secretAccessKey) {
                this.s3Client = new client_s3_1.S3Client({
                    endpoint: this.endpoint,
                    region: this.region,
                    credentials: {
                        accessKeyId: this.accessKeyId,
                        secretAccessKey: this.secretAccessKey,
                    },
                });
            }
        });
    }
    onModuleDestroy() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            return (_a = this.s3Client) === null || _a === void 0 ? void 0 : _a.destroy();
        });
    }
    normalizeDirKey(key) {
        return `${key}/`.replace(/\\+/g, '/').replace(/\/+/g, '/').replace(/^\//, '/');
    }
    normalizeKey(key) {
        return key.replace(/\\+/g, '/').replace(/\/+/g, '/').replace(/^\//, '/').replace(/\/+$/, '');
    }
    mkdir(folderPath, bucket) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.useFileSystem) {
                return fs.promises.mkdir(folderPath, { recursive: true });
            }
            else {
                if (typeof bucket !== 'string') {
                    bucket = this.bucket;
                }
                yield this.s3Client.send(new client_s3_1.PutObjectCommand({ Bucket: bucket, Key: this.normalizeDirKey(folderPath), Body: '', ContentLength: 0 }));
            }
        });
    }
    readdir(folderPath, bucket) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.useFileSystem) {
                return fs.promises.readdir(folderPath);
            }
            else {
                if (typeof bucket !== 'string') {
                    bucket = this.bucket;
                }
                return this.s3Client.send(new client_s3_1.ListObjectsCommand({ Bucket: bucket, Delimiter: '/', Prefix: this.normalizeDirKey(folderPath) })).then((output) => {
                    var _a, _b;
                    const list = [];
                    (_a = output.Contents) === null || _a === void 0 ? void 0 : _a.forEach((content) => {
                        var _a, _b, _c;
                        const key = (_c = (_a = content.Key) === null || _a === void 0 ? void 0 : _a.replace((_b = output.Prefix) !== null && _b !== void 0 ? _b : '', '').replace(/\/+$/, '')) !== null && _c !== void 0 ? _c : '';
                        if (key.length) {
                            list.push(key);
                        }
                    });
                    (_b = output.CommonPrefixes) === null || _b === void 0 ? void 0 : _b.forEach((prefix) => {
                        var _a, _b, _c;
                        const key = (_c = (_a = prefix.Prefix) === null || _a === void 0 ? void 0 : _a.replace((_b = output.Prefix) !== null && _b !== void 0 ? _b : '', '').replace(/\/+$/, '')) !== null && _c !== void 0 ? _c : '';
                        if (key.length) {
                            list.push(key);
                        }
                    });
                    return list.sort();
                });
            }
        });
    }
    rmdir(folderPath, bucket) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.useFileSystem) {
                return fs.promises.rm(folderPath, { recursive: true });
            }
            else {
                if (typeof bucket !== 'string') {
                    bucket = this.bucket;
                }
                const keys = yield this.s3Client.send(new client_s3_1.ListObjectsCommand({ Bucket: bucket, Prefix: this.normalizeDirKey(folderPath) })).then((output) => {
                    var _a;
                    const list = [];
                    (_a = output.Contents) === null || _a === void 0 ? void 0 : _a.forEach((content) => {
                        var _a;
                        list.push((_a = content.Key) !== null && _a !== void 0 ? _a : '');
                    });
                    return list.sort().reverse();
                });
                yield this.s3Client.send(new client_s3_1.DeleteObjectsCommand({ Bucket: bucket, Delete: { Objects: keys.map((key) => ({ Key: key })) } }));
            }
        });
    }
    exists(path, bucket) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.useFileSystem) {
                return fs.existsSync(path);
            }
            else {
                if (typeof bucket !== 'string') {
                    bucket = this.bucket;
                }
                return this.s3Client.send(new client_s3_1.ListObjectsCommand({ Bucket: bucket, Delimiter: '/', Prefix: this.normalizeKey(path) })).then((output) => {
                    if (output.Contents && output.Prefix) {
                        for (const content of output.Contents) {
                            if (content.Key === output.Prefix) {
                                return true;
                            }
                            if (content.Key === output.Prefix + '/') {
                                return true;
                            }
                        }
                    }
                    return false;
                });
            }
        });
    }
    readFile(filePath, bucket) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.useFileSystem) {
                return fs.promises.readFile(filePath);
            }
            else {
                if (typeof bucket !== 'string') {
                    bucket = this.bucket;
                }
                const streamToBuffer = (_stream) => {
                    return new Promise((resolve, reject) => {
                        const chunks = [];
                        _stream.on('data', (chunk) => chunks.push(chunk));
                        _stream.on('error', reject);
                        _stream.on('end', () => resolve(Buffer.concat(chunks)));
                    });
                };
                return this.s3Client.send(new client_s3_1.GetObjectCommand({ Bucket: bucket, Key: this.normalizeKey(filePath) })).then((data) => streamToBuffer(data.Body));
            }
        });
    }
    writeFile(filePath, data, bucket) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.useFileSystem) {
                return fs.promises.writeFile(filePath, data);
            }
            else {
                if (typeof bucket !== 'string') {
                    bucket = this.bucket;
                }
                if (typeof data === 'string') {
                    data = Buffer.from(data);
                }
                yield this.s3Client.send(new client_s3_1.PutObjectCommand({ Bucket: bucket, Key: this.normalizeKey(filePath), Body: data }));
            }
        });
    }
    rm(filePath, bucket) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.useFileSystem) {
                return fs.promises.rm(filePath);
            }
            else {
                if (typeof bucket !== 'string') {
                    bucket = this.bucket;
                }
                yield this.s3Client.send(new client_s3_1.DeleteObjectCommand({ Bucket: bucket, Key: this.normalizeKey(filePath) }));
            }
        });
    }
    createWriteStream(filePath, options = { highWaterMark: 1024 * 1024 * 10 }, bucket) {
        if (options.highWaterMark < 1024 * 1024 * 5) {
            throw new Error('Option: highWaterMark is smaller than the minimum size of 5MB');
        }
        if (this.useFileSystem) {
            return fs.createWriteStream(filePath, options);
        }
        else {
            if (typeof bucket !== 'string') {
                bucket = this.bucket;
            }
            const pass = new stream.PassThrough();
            const upload = new lib_storage_1.Upload({
                client: this.s3Client,
                params: { Bucket: bucket, Key: this.normalizeKey(filePath), Body: pass },
                queueSize: 4,
                partSize: options.highWaterMark,
                leavePartsOnError: false,
            });
            upload.done().then();
            return pass;
        }
    }
};
StorageService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(storage_module_definition_1.MODULE_OPTIONS_TOKEN)),
    __metadata("design:paramtypes", [Object])
], StorageService);
exports.StorageService = StorageService;
